# Prometheus configuration
# Generated by RDT (Rambo Docker Tools)
# Service: {{ service_name }}
# To customise, edit this file and restart the container.
# Existing files are NOT overwritten by RDT on the next run.

global:
  scrape_interval: 15s       # How frequently to scrape targets
  evaluation_interval: 15s   # How frequently to evaluate rules
  scrape_timeout: 10s

# Alertmanager configuration (optional)
# alerting:
#   alertmanagers:
#     - static_configs:
#         - targets: ['alertmanager:9093']

# Rule files (optional)
# rule_files:
#   - "alert_rules.yml"

scrape_configs:
  # Prometheus self-monitoring
  - job_name: 'prometheus'
    static_configs:
      - targets: ['localhost:9090']

  # Add more scrape targets below.
  # Examples:
  #
  # Node Exporter
  # - job_name: 'node-exporter'
  #   static_configs:
  #     - targets: ['node-exporter:9100']
  #
  # cAdvisor (Docker containers metrics)
  # - job_name: 'cadvisor'
  #   static_configs:
  #     - targets: ['cadvisor:8080']
  #
  # Your application (if it exposes /metrics)
  # - job_name: '{{ service_name }}-app'
  #   static_configs:
  #     - targets: ['app:8000']

