# Logstash main configuration
# Generated by RDT (Rambo Docker Tools)
# Service: {{ service_name }}

http.host: "0.0.0.0"
http.port: 9600

pipeline.workers: 2
pipeline.batch.size: 125
pipeline.batch.delay: 50

# Logging
log.level: info
log.format: json

# Monitoring (disabled by default — enable with X-Pack)
xpack.monitoring.enabled: false

