# Filebeat configuration: output → stdout (debug / development)
# Generated by RDT (Rambo Docker Tools)
# Service: {{ service_name }}
# Mode: output-stdout

filebeat.inputs:
  - type: filestream
    id: default-filestream
    enabled: true
    paths:
      - {{ filebeat_log_path | default('/var/log/*.log') }}
    # Add more paths or inputs as needed

output.console:
  pretty: true

processors:
  - add_host_metadata: ~

# Logging
logging.level: info
logging.to_files: false

