# Logstash pipeline: Beats → stdout
# Generated by RDT (Rambo Docker Tools)
# Service: {{ service_name }}
# Mode: beats-stdout (debug / development)

input {
  beats {
    port => 5044
  }
}

filter {
  # Add your filter rules here
  # Example: grok, mutate, date, etc.
}

output {
  stdout {
    codec => rubydebug
  }
}

